@@ -295,9 +295,21 @@ CC_conninfo_init(ConnInfo *conninfo)
 #ifdef	WIN32
 extern	int	platformId;
 #endif /* WIN32 */
+
 /*
  *		IMPLEMENTATION CONNECTION CLASS
  */
+
+static void
+reset_current_schema(ConnectionClass *self)
+{
+	if (self->current_schema)
+	{
+		free(self->current_schema);
+		self->current_schema = NULL;
+	}
+}
+
 ConnectionClass *
 CC_Constructor()
 {
@@ -621,11 +633,7 @@ CC_cleanup(ConnectionClass *self)
 		free(self->server_encoding);
 		self->server_encoding = NULL;
 	}
-	if (self->current_schema)
-	{
-		free(self->current_schema);
-		self->current_schema = NULL;
-	}
+	reset_current_schema(self);
 	/* Free cached table info */
 	if (self->col_info)
 	{
@@ -780,7 +788,7 @@ EatReadyForQuery(ConnectionClass *conn)
 int
 handle_error_message(ConnectionClass *self, char *msgbuf, size_t buflen, char *sqlstate, const char *comment, QResultClass *res)
 {
-	BOOL	new_format = FALSE, msg_truncated = FALSE, truncated;
+	BOOL	new_format = FALSE, msg_truncated = FALSE, truncated, hasmsg = FALSE;
 	SocketClass	*sock = self->sock;
 	char	msgbuffer[ERROR_MSG_LENGTH];
 	UDWORD	abort_opt;
@@ -819,13 +827,23 @@ inolog("new_format=%d\n", new_format);
 					}
 					break;
 				case 'M':
+				case 'D':
 					if (buflen > 0)
 					{
-						strncat(msgbuf, msgbuffer + 1, buflen);
-						buflen -= msgl;
+						if (hasmsg)
+						{
+							strcat(msgbuf, "\n");
+							buflen--;
+						}
+						if (buflen > 0)
+						{
+							strncat(msgbuf, msgbuffer + 1, buflen);
+							buflen -= msgl;
+						}
 					}
 					if (truncated)
 						msg_truncated = truncated;
+					hasmsg = TRUE;
 					break;
 				case 'C':
 					if (sqlstate)
@@ -882,7 +900,7 @@ inolog("new_format=%d\n", new_format);
 int
 handle_notice_message(ConnectionClass *self, char *msgbuf, size_t buflen, char *sqlstate, const char *comment, QResultClass *res)
 {
-	BOOL	new_format = FALSE, msg_truncated = FALSE, truncated;
+	BOOL	new_format = FALSE, msg_truncated = FALSE, truncated, hasmsg = FALSE;
 	SocketClass	*sock = self->sock;
 	char	msgbuffer[ERROR_MSG_LENGTH];
 
@@ -918,15 +936,25 @@ handle_notice_message(ConnectionClass *self, char *msgbuf, size_t buflen, char *
 					}
 					break;
 				case 'M':
+				case 'D':
 					if (buflen > 0)
 					{
-						strncat(msgbuf, msgbuffer + 1, buflen);
-						buflen -= msgl;
+						if (hasmsg)
+						{
+							strcat(msgbuf, "\n");
+							buflen--;
+						}
+						if (buflen > 0)
+						{
+							strncat(msgbuf, msgbuffer + 1, buflen);
+							buflen -= msgl;
+						}
 					}
 					else
 						msg_truncated = TRUE;
 					if (truncated)
 						msg_truncated = truncated;
+					hasmsg = TRUE;
 					break;
 				case 'C':
 					if (sqlstate && !sqlstate[0] && strcmp(msgbuffer + 1, "00000"))
@@ -1067,9 +1095,10 @@ static int	protocol3_opts_array(ConnectionClass *self, const char *opts[][2], BO
 		else
 			opts[cnt++][1] = "on";
 		/* client_encoding */
-		enc = get_environment_encoding(self, "other", NULL);
+		enc = get_environment_encoding(self, self->original_client_encoding, NULL, TRUE);
 		if (enc)
 		{
+			mylog("startup client_encoding=%s\n", enc);
 			opts[cnt][0] = "client_encoding"; opts[cnt++][1] = enc;
 		}
 	}
@@ -1200,10 +1229,34 @@ inolog("return conninfo=%s(%d)\n", conninfo, strlen(conninfo));
 static char CC_initial_log(ConnectionClass *self, const char *func)
 {
 	const ConnInfo	*ci = &self->connInfo;
-	char	*encoding;
+	char	*encoding, vermsg[128];
 
-	qlog("Global Options: Version='%s', fetch=%d, socket=%d, unknown_sizes=%d, max_varchar_size=%d, max_longvarchar_size=%d\n",
-		 POSTGRESDRIVERVERSION,
+	snprintf(vermsg, sizeof(vermsg), "Driver Version='%s,%s'"
+#ifdef	WIN32
+		" linking"
+#ifdef	_MT
+#ifdef	_DLL
+		" dynamic"
+#else
+		" static"
+#endif /* _DLL */
+		" Multithread"
+#else
+		" Singlethread"
+#endif /* _MT */
+#ifdef	NOT_USED
+#ifdef	_DEBUG
+		" Debug"
+#else
+		" Release"
+#endif /* DEBUG */
+#endif /* NOT_USED */
+		" library"
+#endif /* WIN32 */
+		"\n", POSTGRESDRIVERVERSION, PG_BUILD_VERSION);
+	qlog(vermsg);
+	mylog(vermsg);
+	qlog("Global Options: fetch=%d, socket=%d, unknown_sizes=%d, max_varchar_size=%d, max_longvarchar_size=%d\n",
 		 ci->drivers.fetch_max,
 		 ci->drivers.socket_buffersize,
 		 ci->drivers.unknown_sizes,
@@ -1221,13 +1274,13 @@ static char CC_initial_log(ConnectionClass *self, const char *func)
 		 TABLE_NAME_STORAGE_LEN);
 
 	encoding = check_client_encoding(ci->conn_settings);
-	if (encoding && strcmp(encoding, "OTHER"))
-		self->original_client_encoding = strdup(encoding);
+	if (encoding)
+		self->original_client_encoding = encoding;
 	else
 	{
 		encoding = check_client_encoding(ci->drivers.conn_settings);
-		if (encoding && strcmp(encoding, "OTHER"))
-			self->original_client_encoding = strdup(encoding);
+		if (encoding)
+			self->original_client_encoding = encoding;
 	}
 	if (self->original_client_encoding)
 		self->ccsc = pg_CS_code(self->original_client_encoding);
@@ -1395,7 +1448,7 @@ inolog("protocol=%s version=%d,%d\n", ci->protocol, self->pg_version_major, self
 
 	if (!PROTOCOL_62(ci))
 	{
-		BOOL		before_64 = PG_VERSION_LT(self, 6.4),
+		BOOL		beforeV2 = PG_VERSION_LT(self, 6.4),
 					ReadyForQuery = FALSE, retry = FALSE;
 		uint32	leng;
 
@@ -1565,7 +1618,7 @@ inolog("Ekita\n");
 			/*
 			 * There were no ReadyForQuery responce before 6.4.
 			 */
-			if (before_64 && areq == AUTH_REQ_OK)
+			if (beforeV2 && areq == AUTH_REQ_OK)
 				ReadyForQuery = TRUE;
 		} while (!ReadyForQuery);
 	}
@@ -1682,14 +1735,13 @@ inolog("CC_send_settings\n");
 	if (PG_VERSION_GE(self, 6.4))
 	{
 		CC_lookup_characterset(self);
-		if (CC_get_errornumber(self) != 0)
+		if (CC_get_errornumber(self) > 0)
 			return 0;
 #ifdef UNICODE_SUPPORT
 		if (CC_is_in_unicode_driver(self))
 		{
 			if (!self->original_client_encoding ||
-			    (stricmp(self->original_client_encoding, "UNICODE") &&
-			    stricmp(self->original_client_encoding, "UTF8")))
+			    UTF8 != self->ccsc)
 			{
 				QResultClass	*res;
 				if (PG_VERSION_LT(self, 7.1))
@@ -1733,7 +1785,8 @@ inolog("CC_send_settings\n");
 		}
 	}
 
-	CC_clear_error(self);		/* clear any initial command errors */
+	if (CC_get_errornumber(self) > 0)
+		CC_clear_error(self);		/* clear any initial command errors */
 	self->status = CONN_CONNECTED;
 	if (CC_is_in_unicode_driver(self)
 	    && 0 < ci->bde_environment)
@@ -2051,6 +2104,23 @@ mylog("CC_on_abort_partial in\n");
 	CONNLOCK_RELEASE(conn);
 }
 
+static BOOL
+is_setting_search_path(const UCHAR* query)
+{
+	for (query += 4; *query; query++)
+	{
+		if (!isspace(*query))
+		{
+			if (strnicmp(query, "search_path", 11) == 0)
+				return TRUE;
+			query++;
+			while (*query && !isspace(*query))
+				query++;
+		}
+	}
+	return FALSE;
+}
+
 /*
  *	The "result_in" is only used by QR_next_tuple() to fetch another group of rows into
  *	the same existing QResultClass (this occurs when the tuple cache is depleted and
@@ -2081,7 +2151,7 @@ CC_send_query(ConnectionClass *self, char *query, QueryInfo *qi, UDWORD flag, St
 	BOOL		msg_truncated,
 				ReadyToReturn = FALSE,
 				query_completed = FALSE,
-				before_64 = PG_VERSION_LT(self, 6.4),
+				beforeV2 = PG_VERSION_LT(self, 6.4),
 				aborted = FALSE,
 				used_passed_result_object = FALSE,
 			discard_next_begin = FALSE,
@@ -2297,7 +2367,16 @@ inolog("Discarded the first SAVEPOINT\n");
 							res->recent_processed_row_count = atoi(ptr + 1);
 						else
 							res->recent_processed_row_count = -1;
-						 if (!PROTOCOL_74(&(self->connInfo)))
+						if (PROTOCOL_74(&(self->connInfo)))
+						{
+							if (NULL != self->current_schema &&
+							    strnicmp(cmdbuffer, "SET", 3) == 0)
+							{
+								if (is_setting_search_path(query))
+									reset_current_schema(self);
+							}
+						}
+						else
 						{
 							if (strnicmp(cmdbuffer, "COMMIT", 6) == 0)
 								CC_on_commit(self);
@@ -2313,7 +2392,7 @@ inolog("Discarded the first SAVEPOINT\n");
 					QR_set_command(res, cmdbuffer);
 					query_completed = TRUE;
 					mylog("send_query: returning res = %p\n", res);
-					if (!before_64)
+					if (!beforeV2)
 						break;
 
 					/*
@@ -2495,7 +2574,7 @@ inolog("Discarded the first SAVEPOINT\n");
 		/*
 		 * There was no ReadyForQuery response before 6.4.
 		 */
-		if (before_64)
+		if (beforeV2)
 		{
 			if (empty_reqs == 0 && query_completed)
 				break;
@@ -2605,7 +2684,8 @@ CC_send_function(ConnectionClass *self, int fnid, void *result_buf, int *actual_
 	UInt4			leng;
 	Int4			response_length;
 	ConnInfo		*ci;
-	int			func_cs_count = 0; 
+	int			func_cs_count = 0;
+	BOOL			sinceV3, beforeV3, beforeV2, resultResponse;
 
 	mylog("send_function(): conn=%p, fnid=%d, result_is_int=%d, nargs=%d\n", self, fnid, result_is_int, nargs);
 
@@ -2626,7 +2706,10 @@ CC_send_function(ConnectionClass *self, int fnid, void *result_buf, int *actual_
 #define	return DONT_CALL_RETURN_FROM_HERE???
 	ENTER_INNER_CONN_CS(self, func_cs_count);
 	ci = &(self->connInfo);
-	if (PROTOCOL_74(ci))
+	sinceV3 = PROTOCOL_74(ci);
+	beforeV3 = (!sinceV3);
+	beforeV2 = (beforeV3 && !PROTOCOL_64(ci));
+	if (sinceV3)
 	{
 		leng = 4 + sizeof(uint32) + 2 + 2
 			+ sizeof(uint16);
@@ -2657,7 +2740,7 @@ CC_send_function(ConnectionClass *self, int fnid, void *result_buf, int *actual_
 	}
 
 	SOCK_put_int(sock, fnid, 4);
-	if (PROTOCOL_74(ci))
+	if (sinceV3)
 	{
 		SOCK_put_int(sock, 1, 2); /* # of formats */
 		SOCK_put_int(sock, 1, 2); /* the format is binary */
@@ -2680,7 +2763,7 @@ CC_send_function(ConnectionClass *self, int fnid, void *result_buf, int *actual_
 
 	}
 
-	if (PROTOCOL_74(ci))
+	if (sinceV3)
 		SOCK_put_int(sock, 1, 2); /* result format is binary */
 	mylog("    done sending args\n");
 
@@ -2688,6 +2771,7 @@ CC_send_function(ConnectionClass *self, int fnid, void *result_buf, int *actual_
 	mylog("  after flush output\n");
 
 	done = FALSE;
+	resultResponse = FALSE; /* for before V3 only */
 	while (!done)
 	{
 		id = SOCK_get_id(sock);
@@ -2698,16 +2782,21 @@ inolog("send_func response_length=%d\n", response_length);
 		switch (id)
 		{
 			case 'G':
-				if (PROTOCOL_74(ci))
+				if (!resultResponse)
 				{
 					done = TRUE;
 					ret = FALSE;
 					break;
-				}
+				} /* fall through */
 			case 'V':
 				if ('V' == id)
-					if (!PROTOCOL_74(ci))
+				{
+					if (beforeV3) /* FunctionResultResponse */
+					{
+						resultResponse = TRUE;
 						break;
+					}
+				}
 				*actual_result_len = SOCK_get_int(sock, 4);
 				if (-1 != *actual_result_len)
 				{
@@ -2718,12 +2807,14 @@ inolog("send_func response_length=%d\n", response_length);
 
 					mylog("  after get result\n");
 				}
-				if (!PROTOCOL_74(ci))
+				if (beforeV3)
 				{
-					c = SOCK_get_char(sock); /* get the last '0' */ 
+					c = SOCK_get_char(sock); /* get the last '0' */
+					if (beforeV2)
+						done = TRUE;
+					resultResponse = FALSE;
 					mylog("   after get 0\n");
 				}
-				done = TRUE; 
 				break;			/* ok */
 
 			case 'N':
@@ -2740,18 +2831,25 @@ inolog("send_func response_length=%d\n", response_length);
 
 				mylog("send_function(V): 'E' - %s\n", CC_get_errormsg(self));
 				qlog("ERROR from backend during send_function: '%s'\n", CC_get_errormsg(self));
-				done = TRUE;
+				if (beforeV2)
+					done = TRUE;
 				ret = FALSE;
 				break;
 
 			case 'Z':
 				EatReadyForQuery(self);
-				break;
-
-			case '0':			/* empty result */
 				done = TRUE;
 				break;
 
+			case '0':	/* empty result */
+				if (resultResponse)
+				{
+					if (beforeV2)
+						done = TRUE;
+					resultResponse = FALSE;
+					break;
+				} /* fall through */
+
 			default:
 				/* skip the unexpected response if possible */
 				if (response_length >= 0)
@@ -2793,7 +2891,7 @@ CC_setenv(ConnectionClass *self)
  */
 
 	result = PGAPI_AllocStmt(self, &hstmt);
-	if ((result != SQL_SUCCESS) && (result != SQL_SUCCESS_WITH_INFO))
+	if (!SQL_SUCCEEDED(result))
 		return FALSE;
 	stmt = (StatementClass *) hstmt;
 
@@ -2801,7 +2899,7 @@ CC_setenv(ConnectionClass *self)
 
 	/* Set the Datestyle to the format the driver expects it to be in */
 	result = PGAPI_ExecDirect(hstmt, "set DateStyle to 'ISO'", SQL_NTS, 0);
-	if ((result != SQL_SUCCESS) && (result != SQL_SUCCESS_WITH_INFO))
+	if (!SQL_SUCCEEDED(result))
 		status = FALSE;
 
 	mylog("%s: result %d, status %d from set DateStyle\n", func, result, status);
@@ -2809,7 +2907,7 @@ CC_setenv(ConnectionClass *self)
 	if (ci->drivers.disable_optimizer)
 	{
 		result = PGAPI_ExecDirect(hstmt, "set geqo to 'OFF'", SQL_NTS, 0);
-		if ((result != SQL_SUCCESS) && (result != SQL_SUCCESS_WITH_INFO))
+		if (!SQL_SUCCEEDED(result))
 			status = FALSE;
 
 		mylog("%s: result %d, status %d from set geqo\n", func, result, status);
@@ -2820,7 +2918,7 @@ CC_setenv(ConnectionClass *self)
 	if (ci->drivers.ksqo && PG_VERSION_LT(self, 7.1))
 	{
 		result = PGAPI_ExecDirect(hstmt, "set ksqo to 'ON'", SQL_NTS, 0);
-		if ((result != SQL_SUCCESS) && (result != SQL_SUCCESS_WITH_INFO))
+		if (!SQL_SUCCEEDED(result))
 			status = FALSE;
 
 		mylog("%s: result %d, status %d from set ksqo\n", func, result, status);
@@ -2831,7 +2929,7 @@ CC_setenv(ConnectionClass *self)
 	if (PG_VERSION_GT(self, 7.3))
 	{
 		result = PGAPI_ExecDirect(hstmt, "set extra_float_digits to 2", SQL_NTS, 0);
-		if ((result != SQL_SUCCESS) && (result != SQL_SUCCESS_WITH_INFO))
+		if (!SQL_SUCCEEDED(result))
 			status = FALSE;
 
 		mylog("%s: result %d, status %d from set extra_float_digits\n", func, result, status);
@@ -2870,7 +2968,7 @@ CC_send_settings(ConnectionClass *self)
  */
 
 	result = PGAPI_AllocStmt(self, &hstmt);
-	if ((result != SQL_SUCCESS) && (result != SQL_SUCCESS_WITH_INFO))
+	if (!SQL_SUCCEEDED(result))
 		return FALSE;
 	stmt = (StatementClass *) hstmt;
 
@@ -2888,7 +2986,7 @@ CC_send_settings(ConnectionClass *self)
 		while (ptr)
 		{
 			result = PGAPI_ExecDirect(hstmt, ptr, SQL_NTS, 0);
-			if ((result != SQL_SUCCESS) && (result != SQL_SUCCESS_WITH_INFO))
+			if (!SQL_SUCCEEDED(result))
 				status = FALSE;
 
 			mylog("%s: result %d, status %d from '%s'\n", func, result, status, ptr);
@@ -2915,7 +3013,7 @@ CC_send_settings(ConnectionClass *self)
 		while (ptr)
 		{
 			result = PGAPI_ExecDirect(hstmt, ptr, SQL_NTS, 0);
-			if ((result != SQL_SUCCESS) && (result != SQL_SUCCESS_WITH_INFO))
+			if (!SQL_SUCCEEDED(result))
 				status = FALSE;
 
 			mylog("%s: result %d, status %d from '%s'\n", func, result, status, ptr);
@@ -2958,10 +3056,10 @@ CC_lookup_lo(ConnectionClass *self)
 			NULL, IGNORE_ABORT_ON_CONN | ROLLBACK_ON_ERROR, NULL);
 	if (QR_command_maybe_successful(res) && QR_get_num_cached_tuples(res) > 0)
 	{
-		UInt4	basetype;
+		Oid	basetype;
 
-		self->lobj_type = atoi(QR_get_value_backend_row(res, 0, 0));
-		basetype = atoi(QR_get_value_backend_row(res, 0, 1));
+		self->lobj_type = QR_get_value_backend_int(res, 0, 0, NULL);
+		basetype = QR_get_value_backend_int(res, 0, 1, NULL);
 		if (PG_TYPE_OID == basetype)
 			self->lo_is_domain = 1;
 		else if (0 != basetype)
@@ -3033,27 +3131,27 @@ CC_lookup_pg_version(ConnectionClass *self)
  *	has not transitioned to "connected" yet.
  */
 	result = PGAPI_AllocStmt(self, &hstmt);
-	if ((result != SQL_SUCCESS) && (result != SQL_SUCCESS_WITH_INFO))
+	if (!SQL_SUCCEEDED(result))
 		return;
 	stmt = (StatementClass *) hstmt;
 
 	/* get the server's version if possible	 */
 	result = PGAPI_ExecDirect(hstmt, "select version()", SQL_NTS, 0);
-	if ((result != SQL_SUCCESS) && (result != SQL_SUCCESS_WITH_INFO))
+	if (!SQL_SUCCEEDED(result))
 	{
 		PGAPI_FreeStmt(hstmt, SQL_DROP);
 		return;
 	}
 
 	result = PGAPI_Fetch(hstmt);
-	if ((result != SQL_SUCCESS) && (result != SQL_SUCCESS_WITH_INFO))
+	if (!SQL_SUCCEEDED(result))
 	{
 		PGAPI_FreeStmt(hstmt, SQL_DROP);
 		return;
 	}
 
 	result = PGAPI_GetData(hstmt, 1, SQL_C_CHAR, self->pg_version, MAX_INFO_STRING, NULL);
-	if ((result != SQL_SUCCESS) && (result != SQL_SUCCESS_WITH_INFO))
+	if (!SQL_SUCCEEDED(result))
 	{
 		PGAPI_FreeStmt(hstmt, SQL_DROP);
 		return;
@@ -3147,7 +3245,7 @@ CC_get_current_schema(ConnectionClass *conn)
 		if (res = CC_send_query(conn, "select current_schema()", NULL, IGNORE_ABORT_ON_CONN | ROLLBACK_ON_ERROR, NULL), QR_command_maybe_successful(res))
 		{
 			if (QR_get_num_total_tuples(res) == 1)
-				conn->current_schema = strdup(QR_get_value_backend_row(res, 0, 0));
+				conn->current_schema = strdup(QR_get_value_backend_text(res, 0, 0));
 		}
 		QR_Destructor(res);
 	}
@@ -3422,11 +3520,9 @@ LIBPQ_send_cancel_request(const ConnectionClass *conn)
 
 const char *CurrCat(const ConnectionClass *conn)
 {
-	/*
 	if (conn->schema_support)
 		return conn->connInfo.database;
 	else
-	*/
 		return NULL;
 }
 
