@@ -66,6 +66,7 @@ inolog("FI_Constructor reuse=%d\n", reuse);
 	if (reuse)
 		FI_Destructor(&self, 1, FALSE);
 	memset(self, 0, sizeof(FIELD_INFO));
+	self->nullable = TRUE;
 }
 void	FI_Destructor(FIELD_INFO **fi, int count, BOOL freeFI)
 {
@@ -133,7 +134,7 @@ static void IRDFields_free(IRDFields * self)
 	/* Free the parsed field information */
 	if (self->fi)
 	{
-		FI_Destructor(self->fi, self->nfields, TRUE);
+		FI_Destructor(self->fi, self->allocated, TRUE);
 		self->fi = NULL;
 	}
 	self->allocated = 0;
