@@ -5,7 +5,7 @@
  *
  * Comments:		See "notice.txt" for copyright and license information.
  *
- * $Id: descriptor.h,v 1.17 2006/08/24 15:03:55 hinoue Exp $
+ * $Id: descriptor.h,v 1.19 2006/10/13 13:13:31 hinoue Exp $
  *
  */
 
@@ -82,7 +82,7 @@ enum {
 };
 typedef struct
 {
-	UInt4		table_oid;
+	OID		table_oid;
 	COL_INFO	*col_info; /* cached SQLColumns info for this table */
 	pgNAME		schema_name;
 	pgNAME		table_name;
@@ -125,8 +125,9 @@ typedef struct
 	int		column_size; /* precision in 2.x */
 	int		decimal_digits; /* scale in 2.x */
 	int		display_size;
-	int		length;
-	Oid		type;
+	SQLLEN		length;
+	OID		columntype;
+	OID		basetype;
 	char		expr;
 	char		quote;
 	char		dquote;
@@ -138,6 +139,7 @@ Int4 FI_scale(const FIELD_INFO *);
 void	FI_Constructor(FIELD_INFO *, BOOL reuse);
 void	FI_Destructor(FIELD_INFO **, int, BOOL freeFI);
 #define	FI_is_applicable(fi) (NULL != fi && (fi->flag & (FIELD_PARSED_OK | FIELD_COL_ATTRIBUTE)) != 0)
+#define	FI_type(fi) (0 == (fi)->basetype ? (fi)->columntype : (fi)->basetype)
 
 typedef struct DescriptorHeader_
 {
@@ -158,16 +160,16 @@ typedef struct DescriptorHeader_
 struct ARDFields_
 {
 #if (ODBCVER >= 0x0300)
-	int		size_of_rowset; /* for ODBC3 fetch operation */
+	SQLLEN		size_of_rowset; /* for ODBC3 fetch operation */
 #endif /* ODBCVER */
-	int		bind_size;	/* size of each structure if using
+	SQLUINTEGER	bind_size;	/* size of each structure if using
 					 * Row-wise Binding */
-	UInt2		*row_operation_ptr;
+	SQLUSMALLINT	*row_operation_ptr;
 	SQLULEN		*row_offset_ptr;
 	BindInfoClass	*bookmark;
 	BindInfoClass	*bindings;
-	int		allocated;
-	int		size_of_rowset_odbc2; /* for SQLExtendedFetch */
+	SQLSMALLINT	allocated;
+	SQLLEN		size_of_rowset_odbc2; /* for SQLExtendedFetch */
 };
 
 /*
@@ -175,33 +177,37 @@ struct ARDFields_
  */
 struct APDFields_
 {
-	SQLLEN		paramset_size;
-	int		param_bind_type; /* size of each structure if using
+	SQLLEN		paramset_size;	/* really an SQLINTEGER type */ 
+	SQLUINTEGER	param_bind_type; /* size of each structure if using
 					  * Row-wise Parameter Binding */
-	UInt2		*param_operation_ptr;
+	SQLUSMALLINT	*param_operation_ptr;
 	SQLULEN		*param_offset_ptr;
 	ParameterInfoClass	*bookmark; /* dummy item to fit APD to ARD */
 	ParameterInfoClass	*parameters;
-	int		allocated;
-	int		paramset_size_dummy; /* dummy item to fit APD to ARD */
+	SQLSMALLINT	allocated;
+	SQLLEN		paramset_size_dummy; /* dummy item to fit APD to ARD */
 };
 
 struct IRDFields_
 {
 	StatementClass	*stmt;
-	UInt4		*rowsFetched;
-	UInt2		*rowStatusArray;
+	SQLULEN		*rowsFetched;
+	SQLUSMALLINT	*rowStatusArray;
 	UInt4		nfields;
-	UInt4		allocated;
+	SQLSMALLINT	allocated;
 	FIELD_INFO	**fi;
 };
 
 struct IPDFields_
 {
-	UInt4		*param_processed_ptr;
-	UInt2		*param_status_ptr;
+#if (ODBCVER >= 0x0300)
+	SQLUINTEGER		*param_processed_ptr;
+#else
+	SQLULEN			*param_processed_ptr; /* SQLParamOptions */
+#endif /* ODBCVER */
+	SQLUSMALLINT		*param_status_ptr;
+	SQLSMALLINT		allocated;
 	ParameterImplClass	*parameters;
-	int			allocated;
 };
 
 typedef	struct
@@ -248,7 +254,7 @@ BindInfoClass	*ARD_AllocBookmark(ARDFields *self);
 void	ARD_unbind_cols(ARDFields *self, BOOL freeall);
 void	APD_free_params(APDFields *self, char option);
 void	IPD_free_params(IPDFields *self, char option);
-BOOL	getCOLIfromTI(const char *, ConnectionClass *, StatementClass *, const Oid, TABLE_INFO **);
+BOOL	getCOLIfromTI(const char *, ConnectionClass *, StatementClass *, const OID, TABLE_INFO **);
 #if (ODBCVER >= 0x0300)
 RETCODE	DC_set_stmt(DescriptorClass *desc, StatementClass *stmt);
 void	DC_clear_error(DescriptorClass *desc);
