@@ -30,25 +30,25 @@ enum {
 	,LATIN10		/* ISO-8859 Latin 10 */
 	,WIN1256		/* Arabic Windows */
 	,WIN1258		/* Vietnamese Windows */
+	,WIN866			/* Alternativny Variant (MS-DOS CP866) */
 	,WIN874			/* Thai Windows */
 	,KOI8R			/* KOI8-R/U */
-	,WIN1251		/* windows-1251 */
-	,WIN866			/* Alternativny Variant (MS-DOS CP866) */
+	,WIN1251		/* Cyrillic Windows */
+	,WIN1252		/* Western Europe Windows */
 	,ISO_8859_5		/* ISO-8859-5 */
 	,ISO_8859_6		/* ISO-8859-6 */
 	,ISO_8859_7		/* ISO-8859-7 */
 	,ISO_8859_8		/* ISO-8859-8 */
+	,WIN1250		/* Central Europe Windows */
+	,WIN1253		/* Greek Windows */
+	,WIN1254		/* Turkish Windows */
+	,WIN1255		/* Hebrew Windows */
+	,WIN1257		/* Baltic(North Europe) Windows */
 	,SJIS			/* Shift JIS */
 	,BIG5			/* Big5 */
 	,GBK			/* GBK */
 	,UHC			/* UHC */
-	,WIN1250		/* windows-1250 */
 	,GB18030		/* GB18030 */
-	,UNICODE_PODBC		/* same as Unicode UTF-8 */
-	,TCVN			/* TCVN ( < Ver8.1) */
-	,ALT			/* ALT ( < Var8.1) */
-	,WIN			/* WIN ( < Ver8.1) */
-	,WIN1252		/* WIN1252 */
 	,OTHER		=	-1
 };
 
@@ -67,11 +67,11 @@ enum {
 /* New Type */
 
 extern void CC_lookup_characterset(ConnectionClass *self);
-extern const char *get_environment_encoding(const ConnectionClass *conn, const char *oldenc, const char *svrenc);
+extern const char *get_environment_encoding(const ConnectionClass *conn, const char *setenc, const char *svrenc, BOOL bStartup);
 
 extern int pg_CS_stat(int stat,unsigned int charcter,int characterset_code);
 extern int pg_CS_code(const UCHAR *stat_string);
-extern UCHAR *pg_CS_name(int code);
+extern const UCHAR *pg_CS_name(int code);
 
 typedef struct pg_CS
 {
@@ -97,5 +97,6 @@ void encoded_str_constr(encoded_str *encstr, int ccsc, const char *str);
 extern int encoded_nextchar(encoded_str *encstr);
 extern ssize_t encoded_position_shift(encoded_str *encstr, size_t shift);
 extern int encoded_byte_check(encoded_str *encstr, size_t abspos);
-#define check_client_encoding(X) pg_CS_name(pg_CS_code(X))
+/* #define check_client_encoding(X) pg_CS_name(pg_CS_code(X)) */
+UCHAR *check_client_encoding(const UCHAR *sql_string);
 #endif /* __MULTIBUYTE_H__ */
