@@ -5,7 +5,7 @@
  *
  * Comments:		See "notice.txt" for copyright and license information.
  *
- * $Id: psqlodbc.h,v 1.113 2006/10/20 12:00:32 hinoue Exp $
+ * $Id: psqlodbc.h,v 1.118 2007/04/11 16:36:47 h-saito Exp $
  *
  */
 
@@ -119,7 +119,7 @@ typedef	UInt4	OID;
 #ifndef	HAVE_SSIZE_T
 typedef	long	ssize_t
 #endif /* HAVE_SSIZE_T */
-#if (SIZE_OF_VOID_P == SIZE_OF_LONG)
+#if (SIZEOF_VOID_P == SIZEOF_LONG)
 typedef	long 	LONG_PTR;
 typedef	unsigned long 	ULONG_PTR;
 #define	FORMAT_LPTR	"%ld"	/* LONG_PTR */
@@ -131,8 +131,8 @@ typedef	unsigned long long ULONG_PTR;
 #define	FORMAT_ULPTR	"%llu"	/* ULONG_PTR */
 #else
 #error appropriate long pointer type not found 
-#endif /* SIZE_OF_VOID_P */
-#if (SIZE_OF_VOID_P == 8)
+#endif /* SIZEOF_VOID_P */
+#if (SIZEOF_VOID_P == 8)
 #define	FORMAT_INTEGER	"%d"	/* SQLINTEGER */
 #define	FORMAT_UINTEGER	"%u"	/* SQLUINTEGER */
 #if defined(WITH_UNIXODBC) && !defined(BUILD_REAL_64_BIT_MODE)
@@ -147,7 +147,7 @@ typedef	unsigned long long ULONG_PTR;
 #define	FORMAT_ULEN	"%lu"	/* SQLULEN */
 #define	FORMAT_INTEGER	"%ld"	/* SQLINTEGER */
 #define	FORMAT_UINTEGER	"%lu"	/* SQLUINTEGER */
-#endif /* SIZE_OF_VOID_P */
+#endif /* SIZEOF_VOID_P */
 #endif /* WIN32 */
 #define	CAST_PTR(type, ptr)	(type)((LONG_PTR)(ptr))
 #define	CAST_UPTR(type, ptr)	(type)((ULONG_PTR)(ptr))
@@ -303,8 +303,14 @@ typedef double SDOUBLE;
 /*	Info limits */
 #define MAX_INFO_STRING				128
 #define MAX_KEYPARTS				20
-#define MAX_KEYLEN					512 /* max key of the form
+#define MAX_KEYLEN				512 	/* max key of the form
+
 										 * "date+outlet+invoice" */
+/* POSIX defines a PATH_MAX.( wondows is _MAX_PATH ..) */
+#ifndef PATH_MAX
+#define PATH_MAX       1024
+#endif
+
 #define MAX_ROW_SIZE				0	/* Unlimited rowsize with the
 										 * Tuple Toaster */
 #define MAX_STATEMENT_LEN			0	/* Unlimited statement size with
@@ -420,11 +426,12 @@ void		logs_on_off(int cnopen, int, int);
 #define XMIN_ATTNUM				(-3)	/* the attnum of xmin */
 
 /* sizes */
-#define TEXT_FIELD_SIZE				8190		/* size of text fields
-												 * (not including null
-												 * term) */
-#define MAX_VARCHAR_SIZE			254 /* maximum size of a varchar (not
-										 * including null term) */
+#define TEXT_FIELD_SIZE			8190	/* size of default text fields
+						 * (not including null term) */
+#define MAX_VARCHAR_SIZE		255	/* default maximum size of
+						 * varchar fields (not including null term) */
+#define INFO_VARCHAR_SIZE		254	/* varchar field size
+						 * used in info.c */
 
 #define PG_NUMERIC_MAX_PRECISION	1000
 #define PG_NUMERIC_MAX_SCALE		1000
@@ -433,7 +440,7 @@ void		logs_on_off(int cnopen, int, int);
 										 * queries used in info.c inoue
 										 * 2001/05/17 */
 
-#define	LENADDR_SHIFT(x, sft)	((SQLLEN *)((char *)(x) + (sft)))
+#define	LENADDR_SHIFT(x, sft)	((x) ? (SQLLEN *)((char *)(x) + (sft)) : NULL)
 
 int	initialize_global_cs(void);
 #ifdef	POSIX_MULTITHREAD_SUPPORT
