@@ -45,21 +45,21 @@ QR_set_num_fields(QResultClass *self, int new_num_fields)
 
 
 void
-QR_set_position(QResultClass *self, int pos)
+QR_set_position(QResultClass *self, SQLLEN pos)
 {
 	self->tupleField = self->backend_tuples + ((QR_get_rowstart_in_cache(self) + pos) * self->num_fields);
 }
 
 
 void
-QR_set_cache_size(QResultClass *self, int cache_size)
+QR_set_cache_size(QResultClass *self, SQLLEN cache_size)
 {
 	self->cache_size = cache_size;
 }
 
 
 void
-QR_set_rowset_size(QResultClass *self, int rowset_size)
+QR_set_rowset_size(QResultClass *self, Int4 rowset_size)
 {
 	self->rowset_size_include_ommitted = rowset_size;
 }
@@ -73,7 +73,11 @@ QR_set_cursor(QResultClass *self, const char *name)
 	{
 		free(self->cursor_name);
 		if (conn)
+		{
+			CONNLOCK_ACQUIRE(conn);
 			conn->ncursors--;
+			CONNLOCK_RELEASE(conn);
+		}
 		self->cursTuple = -1;
 		self->pstatus = 0;
 	}
@@ -81,7 +85,11 @@ QR_set_cursor(QResultClass *self, const char *name)
 	{
 		self->cursor_name = strdup(name);
 		if (conn)
+		{
+			CONNLOCK_ACQUIRE(conn);
 			conn->ncursors++;
+			CONNLOCK_RELEASE(conn);
+		}
 	}
 	else
 	{
@@ -92,7 +100,7 @@ QR_set_cursor(QResultClass *self, const char *name)
 
 
 void
-QR_set_num_cached_rows(QResultClass *self, int num_rows)
+QR_set_num_cached_rows(QResultClass *self, SQLLEN num_rows)
 {
 	self->num_cached_rows = num_rows;
 	if (QR_synchronize_keys(self))
@@ -100,7 +108,7 @@ QR_set_num_cached_rows(QResultClass *self, int num_rows)
 }
 
 void
-QR_set_rowstart_in_cache(QResultClass *self, int start)
+QR_set_rowstart_in_cache(QResultClass *self, SQLLEN start)
 {
 	if (QR_synchronize_keys(self))
 		self->key_base = start;
@@ -108,7 +116,7 @@ QR_set_rowstart_in_cache(QResultClass *self, int start)
 }
 
 void
-QR_inc_rowstart_in_cache(QResultClass *self, int base_inc)
+QR_inc_rowstart_in_cache(QResultClass *self, SQLLEN base_inc)
 {
 	if (!QR_has_valid_base(self))
 		mylog("QR_inc_rowstart_in_cache called while the cache is not ready\n");
@@ -294,7 +302,7 @@ void
 QR_add_message(QResultClass *self, const char *msg)
 {
 	char	*message = self->message;
-	Int4	alsize, pos;
+	size_t	alsize, pos;
 
 	if (!msg || !msg[0])
 		return;
@@ -329,7 +337,7 @@ void
 QR_add_notice(QResultClass *self, const char *msg)
 {
 	char	*message = self->notice;
-	Int4	alsize, pos;
+	size_t	alsize, pos;
 
 	if (!msg || !msg[0])
 		return;
@@ -353,7 +361,8 @@ QR_add_notice(QResultClass *self, const char *msg)
 
 TupleField	*QR_AddNew(QResultClass *self)
 {
-	UInt4	alloc, num_fields;
+	size_t	alloc;
+	UInt4	num_fields;
 
 	if (!self)	return	NULL;
 inolog("QR_AddNew %dth row(%d fields) alloc=%d\n", self->num_cached_rows, QR_NumResultCols(self), self->count_backend_allocated);
@@ -389,8 +398,7 @@ inolog("QR_AddNew %dth row(%d fields) alloc=%d\n", self->num_cached_rows, QR_Num
 void
 QR_free_memory(QResultClass *self)
 {
-	TupleField	*tuple = self->backend_tuples;
-	int		num_backend_rows = self->num_cached_rows;
+	SQLLEN		num_backend_rows = self->num_cached_rows;
 	int		num_fields = self->num_fields;
 
 	mylog("QResult: free memory in, fcount=%d\n", num_backend_rows);
@@ -413,7 +421,7 @@ QR_free_memory(QResultClass *self)
 		{
 			char	plannm[32];
 
-			sprintf(plannm, "_KEYSET_%x", self);
+			sprintf(plannm, "_KEYSET_%p", self);
 			if (CC_is_in_error_trans(conn))
 			{
 				CC_mark_a_object_to_discard(conn, 's',plannm);
@@ -498,7 +506,7 @@ char
 QR_fetch_tuples(QResultClass *self, ConnectionClass *conn, const char *cursor)
 {
 	CSTR		func = "QR_fetch_tuples";
-	int			tuple_size;
+	SQLLEN			tuple_size;
 
 	/*
 	 * If called from send_query the first time (conn != NULL), then set
@@ -513,7 +521,7 @@ QR_fetch_tuples(QResultClass *self, ConnectionClass *conn, const char *cursor)
 
 		QR_set_conn(self, conn);
 
-		mylog("%s: cursor = '%s', self->cursor=%x\n", func, (cursor == NULL) ? "" : cursor, QR_get_cursor(self));
+		mylog("%s: cursor = '%s', self->cursor=%p\n", func, (cursor == NULL) ? "" : cursor, QR_get_cursor(self));
 
 		if (cursor && !cursor[0])
 			cursor = NULL;
@@ -645,7 +653,7 @@ QR_close(QResultClass *self)
 		/* End the transaction if there are no cursors left on this conn */
 		if (CC_is_in_autocommit(conn) && CC_cursor_count(conn) == 0)
 		{
-			mylog("QResult: END transaction on conn=%x\n", conn);
+			mylog("QResult: END transaction on conn=%p\n", conn);
 
 			if (!CC_commit(conn))
 			{
@@ -664,7 +672,7 @@ BOOL
 QR_get_tupledata(QResultClass *self, BOOL binary)
 {
 	BOOL	haskeyset = QR_haskeyset(self);
-	UInt4	num_total_rows = QR_get_num_total_tuples(self);
+	SQLULEN num_total_rows = QR_get_num_total_tuples(self);
 
 inolog("QR_get_tupledata num_fields=%d\n", self->num_fields);
 	if (!QR_get_cursor(self))
@@ -673,7 +681,7 @@ inolog("QR_get_tupledata num_fields=%d\n", self->num_fields);
 		if (self->num_fields > 0 &&
 		    num_total_rows >= self->count_backend_allocated)
 		{
-			int	tuple_size = self->count_backend_allocated;
+			SQLLEN	tuple_size = self->count_backend_allocated;
 
 			mylog("REALLOC: old_count = %d, size = %d\n", tuple_size, self->num_fields * sizeof(TupleField) * tuple_size);
 			if (tuple_size < 1)
@@ -686,7 +694,7 @@ inolog("QR_get_tupledata num_fields=%d\n", self->num_fields);
 		if (haskeyset &&
 		    self->num_cached_keys >= self->count_keyset_allocated)
 		{
-			int	tuple_size = self->count_keyset_allocated;
+			SQLLEN	tuple_size = self->count_keyset_allocated;
 
 			if (tuple_size < 1)
 				tuple_size = TUPLE_MALLOC_INC;
@@ -703,7 +711,7 @@ inolog("QR_get_tupledata num_fields=%d\n", self->num_fields);
 		QR_set_message(self, "Error reading the tuple");
 		return FALSE;
 	}
-inolog("!!%x->cursTup=%d total_read=%d\n", self, self->cursTuple, self->num_total_read);
+inolog("!!%p->cursTup=%d total_read=%d\n", self, self->cursTuple, self->num_total_read);
 	if (!QR_once_reached_eof(self) && self->cursTuple >= (Int4) self->num_total_read)
 		self->num_total_read = self->cursTuple + 1;
 inolog("!!cursTup=%d total_read=%d\n", self->cursTuple, self->num_total_read);
@@ -717,9 +725,10 @@ inolog("!!cursTup=%d total_read=%d\n", self->cursTuple, self->num_total_read);
 	return TRUE;
 }
 
-static int enlargeKeyCache(QResultClass *self, Int4 add_size, const char *message)
+static SQLLEN enlargeKeyCache(QResultClass *self, SQLLEN add_size, const char *message)
 {
-	Int4	alloc, alloc_req, num_fields = self->num_fields;
+	size_t	alloc, alloc_req;
+	Int4	num_fields = self->num_fields;
 	BOOL	curs = (NULL != QR_get_cursor(self));
 
 	if (add_size <= 0)
@@ -778,11 +787,11 @@ QR_next_tuple(QResultClass *self, StatementClass *stmt)
 	SocketClass *sock;
 
 	/* Speed up access */
-	int		fetch_number = self->fetch_number, cur_fetch;
-	Int4		num_total_rows;
-	Int4		num_backend_rows = self->num_cached_rows,
-			num_fields = self->num_fields, num_rows_in;
-	Int4		req_size, fetch_size, offset = 0,	end_tuple;
+	SQLLEN		fetch_number = self->fetch_number, cur_fetch = 0;
+	SQLLEN		num_total_rows;
+	SQLLEN		num_backend_rows = self->num_cached_rows, num_rows_in;
+	Int4		num_fields = self->num_fields, fetch_size, req_size;
+	SQLLEN		offset = 0, end_tuple;
 	char		boundary_adjusted = FALSE;
 	TupleField *the_tuples = self->backend_tuples;
 
@@ -797,13 +806,15 @@ QR_next_tuple(QResultClass *self, StatementClass *stmt)
 	ConnInfo   *ci = NULL;
 	BOOL		msg_truncated, rcvend, internally_invoked = FALSE;
 	BOOL		reached_eof_now = FALSE, curr_eof; /* detecting EOF is pretty important */
+	BOOL		ExecuteRequest = FALSE;
 	Int4		response_length;
 
-inolog("Oh %x->fetch_number=%d\n", self, self->fetch_number);
+inolog("Oh %p->fetch_number=%d\n", self, self->fetch_number);
 inolog("in total_read=%d cursT=%d currT=%d ad=%d total=%d rowsetSize=%d\n", self->num_total_read, self->cursTuple, stmt ? stmt->currTuple : -1, self->ad_count, QR_get_num_total_tuples(self), self->rowset_size_include_ommitted);
 
 	num_total_rows = QR_get_num_total_tuples(self);
 	conn = QR_get_conn(self);
+	curr_eof = FALSE;
 	req_size = self->rowset_size_include_ommitted;
 	if (QR_once_reached_eof(self) && self->cursTuple >= (Int4) QR_get_num_total_read(self))
 		curr_eof = TRUE;
@@ -811,14 +822,14 @@ inolog("in total_read=%d cursT=%d currT=%d ad=%d total=%d rowsetSize=%d\n", self
 	{
 		char		movecmd[256];
 		QResultClass	*mres;
-		UInt4		movement, moved;
+		SQLULEN		movement, moved;
 
 		movement = self->move_offset;
 		if (QR_is_moving_backward(self))
 		{
 			if (self->cache_size > req_size)
 			{
-				int	incr_move = self->cache_size - (req_size < 0 ? 1 : req_size);
+				SQLLEN	incr_move = self->cache_size - (req_size < 0 ? 1 : req_size);
 				
 				movement += incr_move;
 				if (movement > (UInt4)(self->cursTuple + 1))
@@ -826,11 +837,11 @@ inolog("in total_read=%d cursT=%d currT=%d ad=%d total=%d rowsetSize=%d\n", self
 			}
 			else
 				self->cache_size = req_size;
-inolog("cache=%d rowset=%d movement=%d\n", self->cache_size, req_size, movement);
-			sprintf(movecmd, "move backward %u in \"%s\"", movement, QR_get_cursor(self));
+inolog("cache=%d rowset=%d movement=" FORMAT_ULEN "\n", self->cache_size, req_size, movement);
+			sprintf(movecmd, "move backward " FORMAT_ULEN " in \"%s\"", movement, QR_get_cursor(self));
 		}
 		else if (QR_is_moving_forward(self))
-			sprintf(movecmd, "move %u in \"%s\"", movement, QR_get_cursor(self));
+			sprintf(movecmd, "move " FORMAT_ULEN " in \"%s\"", movement, QR_get_cursor(self));
 		else
 		{
 			sprintf(movecmd, "move all in \"%s\"", QR_get_cursor(self));
@@ -845,9 +856,9 @@ inolog("cache=%d rowset=%d movement=%d\n", self->cache_size, req_size, movement)
 			return -1;
 		}
 		moved = movement;
-		if (sscanf(mres->command, "MOVE %u", &moved) > 0)
+		if (sscanf(mres->command, "MOVE %lu", &moved) > 0)
 		{
-inolog("moved=%d ? %d\n", moved, movement);
+inolog("moved=%d ? " FORMAT_ULEN "\n", moved, movement);
         		if (moved < movement)
 			{
 				if (0 <  moved)
@@ -869,8 +880,8 @@ inolog("moved=%d ? %d\n", moved, movement);
 				}
 				if (QR_is_moving_from_the_last(self))  /* in case of FETCH LAST */
 				{
-					UInt4	bmovement, mback;
-					Int4	rowset_start = self->cursTuple + 1, back_offset = self->move_offset, backpt;
+					SQLULEN	bmovement, mback;
+					SQLLEN	rowset_start = self->cursTuple + 1, back_offset = self->move_offset, backpt;
 inolog("FETCH LAST case\n");
 					if (getNthValid(self, QR_get_num_total_tuples(self) - 1, SQL_FETCH_PRIOR, self->move_offset, &backpt) < 0)
 					{
@@ -885,7 +896,7 @@ inolog("back_offset=%d and move_offset=%d\n", back_offset, self->move_offset);
 					if (back_offset + 1 > (Int4) self->ad_count)
 					{
 						bmovement = back_offset + 1 - self->ad_count;
-						sprintf(movecmd, "move backward %u in \"%s\"", bmovement, QR_get_cursor(self));
+						sprintf(movecmd, "move backward " FORMAT_ULEN " in \"%s\"", bmovement, QR_get_cursor(self));
 						QR_Destructor(mres);
 						mres = CC_send_query(conn, movecmd, NULL, 0, stmt);
 						if (!QR_command_maybe_successful(mres))
@@ -896,7 +907,7 @@ inolog("back_offset=%d and move_offset=%d\n", back_offset, self->move_offset);
 							return -1;
 						}
 
-						if (sscanf(mres->command, "MOVE %u", &mback) > 0)
+						if (sscanf(mres->command, "MOVE %lu", &mback) > 0)
 						{
 							if (mback < bmovement)
 								mback++;
@@ -955,7 +966,7 @@ inolog("back_offset=%d and move_offset=%d\n", back_offset, self->move_offset);
 		/* return a row from cache */
 		mylog("%s: fetch_number < fcount: returning tuple %d, fcount = %d\n", func, fetch_number, num_backend_rows);
 		self->tupleField = the_tuples + (fetch_number * num_fields);
-inolog("tupleField=%x\n", self->tupleField);
+inolog("tupleField=%p\n", self->tupleField);
 		/* move to next row */
 		QR_inc_next_in_cache(self);
 		return TRUE;
@@ -963,7 +974,7 @@ inolog("tupleField=%x\n", self->tupleField);
 	else if (QR_once_reached_eof(self))
 	{
 		BOOL	reached_eod = FALSE;
-		UInt4	num_total_read = self->num_total_read;
+		SQLULEN	num_total_read = self->num_total_read;
 
 		if (stmt)
 		{
@@ -992,6 +1003,7 @@ inolog("tupleField=%x\n", self->tupleField);
 	 */
 	self->tupleField = NULL;
 
+	fetch_size = 0;
 	if (!QR_is_fetching_tuples(self))
 	{
 		ci = &(conn->connInfo);
@@ -1034,7 +1046,7 @@ inolog("clear obsolete %d tuples\n", num_backend_rows);
 			 *	and fetch the rest of the rowset.
 			 */
 			/* The next fetch size is */
-			fetch_size = end_tuple - num_backend_rows;
+			fetch_size = (Int4) (end_tuple - num_backend_rows);
 			if (fetch_size <= 0)
 			{
 				mylog("corrupted fetch_size end_tuple=%d <= cached_rows=%d\n", end_tuple, num_backend_rows);
@@ -1049,8 +1061,11 @@ inolog("clear obsolete %d tuples\n", num_backend_rows);
 
 		if (enlargeKeyCache(self, self->cache_size - num_backend_rows, "Out of memory while reading tuples") < 0)
 			return FALSE;
-		if (PROTOCOL_74(ci))
+		if (PROTOCOL_74(ci)
+		    && !QR_is_permanent(self) /* Execute seems an invalid operation after COMMIT */ 
+			)
 		{
+			ExecuteRequest = TRUE;
 			if (!SendExecuteRequest(stmt, QR_get_cursor(self),
 				fetch_size))
 				return FALSE;
@@ -1108,6 +1123,8 @@ inolog("reached_eof_now=%d\n", reached_eof_now);
 	for (rcvend = FALSE; !rcvend;)
 	{
 		id = SOCK_get_id(sock);
+		if (0 != SOCK_get_errcode(sock))
+			break;
 		response_length = SOCK_get_response_length(sock);
 inolog("id='%c' response_length=%d\n", id, response_length);
 		switch (id)
@@ -1164,7 +1181,7 @@ inolog("id='%c' response_length=%d\n", id, response_length);
 				SOCK_get_string(sock, cmdbuffer, ERROR_MSG_LENGTH);
 				QR_set_command(self, cmdbuffer);
 
-				mylog("end of tuple list -- setting inUse to false: this = %x %s\n", self, cmdbuffer);
+				mylog("end of tuple list -- setting inUse to false: this = %p %s\n", self, cmdbuffer);
 
 				qlog("    [ fetched %d rows ]\n", self->num_total_read);
 				mylog("_%s: 'C' fetch_total = %d & this_fetch = %d\n", func, self->num_total_read, self->num_cached_rows);
@@ -1173,7 +1190,7 @@ inolog("id='%c' response_length=%d\n", id, response_length);
 					QR_set_no_fetching_tuples(self);
 					if (internally_invoked)
 					{
-						if (PROTOCOL_74(ci)) /* Execute completed without accepting Portal Suspend */
+						if (ExecuteRequest) /* Execute completed without accepting Portal Suspend */
 							reached_eof_now = TRUE;
 						else if (cur_fetch < fetch_size)
 							reached_eof_now = TRUE;
@@ -1236,11 +1253,25 @@ inolog("id='%c' response_length=%d\n", id, response_length);
 				ret = FALSE;
 				rcvend = TRUE;
 		}
+		if (0 != SOCK_get_errcode(sock))
+			break;
 	}
+	if (0 != SOCK_get_errcode(sock))
+	{
+		if (QR_command_maybe_successful(self))
+		{
+			QR_set_message(self, "Communication error while getting a tuple");
+			QR_set_rstatus(self, PORES_FATAL_ERROR);
+		}
+		CC_on_abort(conn, CONN_DEAD);
+		ret = FALSE;
+	}
+	if (!ret)
+		return ret;
 
 	if (!QR_is_fetching_tuples(self))
 	{
-		Int4	start_idx = 0;
+		SQLLEN	start_idx = 0;
 
 		num_backend_rows = self->num_cached_rows;
 		if (reached_eof_now)
@@ -1253,7 +1284,7 @@ inolog("id='%c' response_length=%d\n", id, response_length);
 			    cur_fetch < fetch_size)
 			{
 				/* We have to append the tuples(keys) info from the added tuples(keys) here */
-				Int4 add_size;
+				SQLLEN	add_size;
 				TupleField	*tuple, *added_tuple;
 
 				if (curr_eof)
@@ -1312,9 +1343,9 @@ inolog("will add %d added_tuples from %d and select the %dth added tuple\n", add
 	*/
 	if (internally_invoked && self->keyset && (self->dl_count > 0 || self->up_count > 0))
 	{
-		int	i, lf;
-		Int4	lidx, hidx;
-		UInt4	*deleted = self->deleted, *updated = self->updated;
+		SQLLEN	i, lf;
+		SQLLEN	lidx, hidx;
+		SQLULEN	*deleted = self->deleted, *updated = self->updated;
  
 		num_backend_rows = QR_get_num_cached_tuples(self);
 		/* For simplicty, use CURS_NEEDS_REREAD bit to mark the row */
@@ -1403,9 +1434,9 @@ QR_read_a_tuple_from_db(QResultClass *self, char binary)
 	{
 		int	numf = SOCK_get_int(sock, sizeof(Int2));
 if (effective_cols > 0)
-inolog("%dth record in cache numf=%d\n", self->num_cached_rows, numf);
+{inolog("%dth record in cache numf=%d\n", self->num_cached_rows, numf);}
 else
-inolog("%dth record in key numf=%d\n", self->num_cached_keys, numf);
+{inolog("%dth record in key numf=%d\n", self->num_cached_keys, numf);}
 	}
 	else
 		SOCK_get_n_char(sock, bitmap, bitmaplen);
@@ -1459,7 +1490,7 @@ inolog("QR_read_a_tuple_from_db len=%d\n", len);
 			if (field_lf >= effective_cols)
 			{
 				if (field_lf == effective_cols)
-					sscanf(buffer, "(%lu,%hu)",
+					sscanf(buffer, "(%u,%hu)",
 						&this_keyset->blocknum, &this_keyset->offset);
 				else
 					this_keyset->oid = strtoul(buffer, NULL, 10);
