@@ -8,11 +8,18 @@
  */
 
 #include "psqlodbc.h"
-#include <stdio.h>
-#include <string.h>
-#ifdef	WIN32
+#ifdef  WIN32
+/* gcc <malloc.h> has been replaced by <stdlib.h> */
 #include <malloc.h>
+#ifdef  _DEBUG
+#ifndef _MEMORY_DEBUG_
+#include <stdlib.h>
+#define _CRTDBG_MAP_ALLOC
+#include <crtdbg.h>
+#endif /* _MEMORY_DEBUG_ */
+#endif /* _DEBUG */
 #endif /* WIN32 */
+#include <string.h>
 
 #define	byte3check	0xfffff800
 #define	byte2_base	0x80c0
@@ -23,12 +30,12 @@
 #define	byte3_mask2	0x0fc0
 #define	byte3_mask3	0x003f
 
+#include <ctype.h>
 #ifndef WIN32
 #ifdef HAVE_ISWASCII
 #include <wctype.h>
 #else
 #include <wchar.h>
-#include <ctype.h>
 int	iswascii(wchar_t c)
 {
 	return isascii(wctob(c));
@@ -36,17 +43,17 @@ int	iswascii(wchar_t c)
 #endif  /* HAVE_ISWASCII */
 #endif  /* WIN32 */
 
-UInt4	ucs2strlen(const SQLWCHAR *ucs2str)
+SQLULEN	ucs2strlen(const SQLWCHAR *ucs2str)
 {
-	UInt4	len;
+	SQLULEN	len;
 	for (len = 0; ucs2str[len]; len++)
 		;
 	return len;
 }
-char *ucs2_to_utf8(const SQLWCHAR *ucs2str, Int4 ilen, Int4 *olen, BOOL lower_identifier)
+char *ucs2_to_utf8(const SQLWCHAR *ucs2str, SQLLEN ilen, SQLLEN *olen, BOOL lower_identifier)
 {
 	char *	utf8str;
-/*mylog("ucs2_to_utf8 %x ilen=%d ", ucs2str, ilen);*/
+/*mylog("ucs2_to_utf8 %p ilen=%d ", ucs2str, ilen);*/
 
 	if (!ucs2str)
 	{
@@ -106,10 +113,10 @@ char *ucs2_to_utf8(const SQLWCHAR *ucs2str, Int4 ilen, Int4 *olen, BOOL lower_id
 #define	byte3_m3	0x3f
 #define	byte2_m1	0x1f
 #define	byte2_m2	0x3f
-UInt4	utf8_to_ucs2_lf(const char *utf8str, Int4 ilen, BOOL lfconv, SQLWCHAR *ucs2str, UInt4 bufcount)
+SQLULEN	utf8_to_ucs2_lf(const char *utf8str, SQLLEN ilen, BOOL lfconv, SQLWCHAR *ucs2str, SQLULEN bufcount)
 {
 	int	i;
-	UInt4	ocount, wcode;
+	SQLULEN	ocount, wcode;
 	const UCHAR *str;
 
 /*mylog("utf8_to_ucs2 ilen=%d bufcount=%d", ilen, bufcount);*/
@@ -122,9 +129,10 @@ UInt4	utf8_to_ucs2_lf(const char *utf8str, Int4 ilen, BOOL lfconv, SQLWCHAR *ucs
 		bufcount = 0;
 	if (ilen < 0)
 		ilen = strlen(utf8str);
-	for (i = 0, ocount = 0, str = utf8str; i < ilen;)
+	for (i = 0, ocount = 0, str = utf8str; i < ilen && *str;)
 	{
-		if (iswascii(*str))
+		/* if (iswascii(*str)) */
+		if (isascii(*str))
 		{
 			if (lfconv && PG_LINEFEED == *str &&
 			    (i == 0 || PG_CARRIAGE_RETURN != str[-1]))
